<?php

/**
 * Framing sample
 *
 * http://michael.tyson.id.au/advanced-web-templating-with-php-and-regular-expressions
 *
 * Copyright (C) 2008 Michael Tyson <mike@tyson.id.au>
 * This code is licensed under the GPL: http://www.opensource.org/licenses/gpl-license.php
 */


/**
 * Constants to configure image sizing
 */
define("FRAME_MAX_WIDTH", 415);
define("FRAME_SMALL_WIDTH", 140);
define("FRAME_SMALL_HEIGHT", 140);
define("FRAME_MIN_WIDTH", 25);
define("FRAME_MIN_HEIGHT", 25);

/**
 * Add frames
 *
 * Filters content, adding frames to all objects with class 'frame', and all images if
 *	corresponding option is set.
 */
function add_frame_filter( $content , $arg2=null, $arg3=null ) {
	global $post;

	// Not for feeds
	if ( is_feed() ) return $content;
	
	// Look-ahead for class 'frame'
	$frameClass = '(?=[^>]+class=["\'][^"\']*frame)';
	
	// Skipped classes
	$classes = "(?:".join("|", array_map("trim", explode(",",get_option('frame_class_skip')))).")";
	$skippedClasses = '(?![^>]+class=["\'][^"\']*'.$classes.')';
	
	// 'A' tags which we want to include inside the frame
	$aStart = '(?:<\s*a[^>]+>\s*)?';
	$aEnd = '(?:\s*</a\s*>)?';

	// Beginning tag, including class check
	$startTagWithFrameClass = "<\s*((?:(img)|[a-zA-Z]+))${skippedClasses}${frameClass}[^>]*";
	
	// End of tag: Short form
	$endSingleTag = '(?(2)\s*/?>|\s*/>)';
	
	// End of tag: Long form
	$endOriginalTag = '</\s*\\g{1}\s*>';

	// Any tag
	$anyStartTag = '<\s*([a-z]+)[^>]*';
	$endLastTag = '</\s*\\g{-1}\s*>';
	
	// Nested content - tags within tags
	$nestedContent = "([^<]+|(?:$anyStartTag(?:$endSingleTag|>(?-2)*$endLastTag)))*";

	// Composite expression - look for a block with class of 'frame', and include all content
	$regex = "$aStart$startTagWithFrameClass(?:$endSingleTag|>$nestedContent$endOriginalTag)$aEnd";
	$regexes = array("@".$regex."@is");

	// Also replace all images
	$frame_all_images = get_option("frame_all_images");
	$frame_all_images_post = get_post_meta($post->ID, 'frame_all_images', true);
	if ( $frame_all_images_post == "true" )
		$frame_all_images = true;
	else if ( $frame_all_images_post == "false" )
		$frame_all_images = false;
		
	if ( $frame_all_images ) {
		
		// Look-ahead for not class of frame (becuase we caught these above)
		$notFrameClass = '(?![^>]+class=["\'][^"\']*frame)';
		
		// Composite expression - any images not with class of 'frame'
		$regex = "$aStart\s*<\s*(img)${notFrameClass}[^>]+>\s*$aEnd";
		$regexes[] = "@".$regex."@is";
	}
	
	// Perform replacement with helper function below
	$newContent = @preg_replace_callback($regexes, 'frame_replace_helper', $content);
	if ( !$newContent ) {
		return $content;
	}
	
	return $newContent;
}


/**
 * Filter helper function
 *
 *	Perform replacements for blocks discovered in the filter function above.
 *	Adds surrounding divs for frame, styled according to original block,
 * resizes too-large images, and ignores entire block if it is a too-small 
 * image (because it will look weird, otherwise)
 */
function frame_replace_helper($matches) {

	$inner = $matches[0];
	
	// Look for align and style attributes
	$align = '(^(?:<\s*a[^>]+>)?\s*<[^>]*?)align=["\'](left|right)["\']';
	$style = '(^(?:<\s*a[^>]+>)?\s*<[^>]*?)style=["\']([^"\']+)["\']';
	$class = '(^(?:<\s*a[^>]+>)?\s*<[^>]*?)class=["\']([^"\']+)["\']';

	$styles = "";
	if ( preg_match( "@$align@is", $inner, $newmatch ) ) {
		// Align attribute found: Add an equivalent float
		$styles .= "float: ".$newmatch[2].";";
	}
	
	if ( preg_match( "@$style@is", $inner, $newmatch ) ) {
		// Style attribute found: Remember content, but minus some undesirable elements
		$styles .= preg_replace("@(border(-[a-z]+)?|padding(-[a-z]+)?|margin-([a-z]+)?)\s*:[^;\"']*@is", "", $newmatch[2]);
	}
	
	if ( preg_match( "@$class@is", $inner, $newmatch ) ) {
		// Style attribute found: Remember content
		$classes = trim(preg_replace("@(?<![a-z])frame(?!=[a-z])@is", "", $newmatch[2]));
	}
	
	// Check width and height
	$widthRegex = '@(^(?:<\s*a[^>]+>)?\s*<[^>]*?(?:width=[\'"]|width:\s*))([0-9]+)@is';
	$heightRegex = '@(^(?:<\s*a[^>]+>)?\s*<[^>]*?(?:height=[\'"]|height:\s*))([0-9]+)@is';

	if ( preg_match( $widthRegex, $inner, $newmatch ) ) {
		$width = $newmatch[2];
	}
	if ( preg_match( $heightRegex, $inner, $newmatch ) ) {
		$height = $newmatch[2];
	}

	if ( ( $width && $width < FRAME_MIN_WIDTH ) || ( $height && $height < FRAME_MIN_HEIGHT ) ) {
		// Image is too small - just skip this one: return original content
		return $inner;
	} 
	
	if ( $width && $width > FRAME_MAX_WIDTH ) {
		// Image is too large - scale down proportionately
		if ( $height ) {
			$ratio = $width / $height;
			$height = round(FRAME_MAX_WIDTH / $ratio);
			
			// Replace height value
			$inner = preg_replace( $heightRegex, "\${1}$height", $inner );
		}
		$width = FRAME_MAX_WIDTH;
		
		// Replace width value
		$inner = preg_replace( $widthRegex, "\${1}$width", $inner );	
	}
	
	$small = '';
	if ( ( $width && $width < FRAME_SMALL_WIDTH ) || ( $height && $height < FRAME_SMALL_HEIGHT ) ) {
		// Image is too small for the large frame - use the small frame
		$small = ' small';
	}
	
	// Wrap content, and remove align/style from inner tag
	return '<span class="frame-outer '.$small.' '.$classes.'"'.($styles ? ' style="'.$styles.'"' : '' ).'><span><span><span>' .
					preg_replace(array("@${align}@is","@${style}@is"), '\\1', $inner).
				'</span></span></span></span>';
}


// Use our filter to modify content
add_filter( 'the_content', 'add_frame_filter' );

// Set up some options
add_option( 'frame_all_images', true );
add_option( 'frame_class_skip', 'noframe,wp-smiley' );

?>